#pragma once

#include <QGeoCoordinate>
#include <QObject>

#include <memory>

#include "IDArray.h"
#include "LogicalArray.h"
#include "TileArray.h"
#include "TilePtrArray.h"

// Singelton class used to interface measurement devices implementing the nemo
// interface.
class NemoInterface : public QObject {
  Q_OBJECT
  class Impl;
  using PImpl = std::unique_ptr<Impl>;

  NemoInterface();
  NemoInterface(NemoInterface &other) = delete;
  static NemoInterface *createInstance();

public:
  ~NemoInterface() override;
  static NemoInterface *instance();

  enum class STATUS {
    NOT_CONNECTED = 0,
    HEARTBEAT_DETECTED = 1,
    WEBSOCKET_DETECTED = 2,
    TIMEOUT = -1,
    INVALID_HEARTBEAT = -2
  };
  Q_ENUM(STATUS)

  Q_PROPERTY(STATUS status READ status NOTIFY statusChanged)
  Q_PROPERTY(QString statusString READ statusString NOTIFY statusChanged)
  Q_PROPERTY(QString editorQml READ editorQml CONSTANT)
  Q_PROPERTY(bool running READ running NOTIFY runningChanged)

  Q_INVOKABLE void start();
  Q_INVOKABLE void stop();

  void addTiles(const TilePtrArray &tileArray);
  void addTiles(const TileArray &tileArray);
  void removeTiles(const IDArray &idArray);
  void clearTiles();
  TileArray getTiles(const IDArray &idArray);
  TileArray getAllTiles();
  LogicalArray containsTiles(const IDArray &idArray);
  TileArray extractTiles(const IDArray &idArray);
  std::size_t size();
  bool empty();

  STATUS status() const;
  QString statusString() const;
  QString editorQml();
  bool running();

signals:
  void statusChanged();
  void progressChanged();
  void runningChanged();

private:
  PImpl pImpl;
};

#define pNemoInterface NemoInterface::instance()
